using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using ScriptCallbacks.BusinessObjects;

public partial class Anthem_CustomerListControls : System.Web.UI.Page
{

    protected busCustomer Customer = null;

    /// <summary>
    /// Note I'm binding in OnInit due to no ViewState and SelectedIndex
    /// issues. List must be bound before values are read from Post Data
    /// </summary>
    /// <param name="e"></param>
    protected override void OnInit(EventArgs e)
    {
        Customer = new busCustomer();

        // *** Bind the list early so SelectedValue can be assigned
        this.BindCustomerList(); 
        
        base.OnInit(e);
    }

    #region Handle SelectedIndex in non-viewstate environment so SelectedIndexChanged isn't always fired
    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);
        this.ViewState["lstCustomers_SelectedValue"] = this.lstCustomers.SelectedValue;
    }

    protected override void LoadViewState(object savedState)
    {
        base.LoadViewState(savedState);
        string Val = this.ViewState["lstCustomers_SelectedValue"] as string;
        if (Val != null)
            this.lstCustomers.SelectedValue = Val;
    }
    #endregion


    protected void Page_Load(object sender, EventArgs e)
    {
        Anthem.Manager.Register(this);

        if (!this.IsPostBack && this.lstCustomers.Items.Count > 0)
        {
            this.lstCustomers.SelectedValue = this.lstCustomers.Items[0].Value;
            this.BindCustomerInfo(this.lstCustomers.Items[0].Value);
        }
    }


    protected void BindCustomerList()
    {
        this.Customer.GetCustomerList();

        this.lstCustomers.DataSource = this.Customer.DataSet.Tables["TCustomerList"];
        this.lstCustomers.DataValueField = "CustomerId";
        this.lstCustomers.DataTextField = "CompanyName";
        this.lstCustomers.DataBind();
    }

    protected void BindCustomerInfo(string CustomerId)
    {
        if (!this.Customer.Load(CustomerId))
            new ApplicationException("Invalid Customer selected.");

        this.CustomerInfoUpdatePanel.DataBind();

        string value = this.txtCompany.Text;
        txtAddress.Text = "West Wind";

        if (Anthem.Manager.IsCallBack)
            this.CustomerInfoUpdatePanel.UpdateAfterCallBack = true;
    }

    protected void BindInvoiceList(string CustomerId)
    {
        busInvoice Invoice = new busInvoice();
        if (Invoice.GetInvoicesForCustomer(CustomerId) < 0)
        {
            this.ShowMessage("Error: " + Invoice.ErrorMessage);
            this.CustomerInfoUpdatePanel.UpdateAfterCallBack = true;
            return;
        }

        this.dgOrders.DataSource = Invoice.DataSet.Tables["TCustomerInvoiceList"];
        
        // *** Calculate the total to display in the footer
        decimal Totals = 0.00M;
        foreach (DataRow dr in Invoice.DataSet.Tables["TCustomerInvoiceList"].Rows)
        {
            Totals += (decimal) dr["OrderTotal"];
        }

        this.dgOrders.Columns[1].FooterText = "&nbsp;Total for all invoices:";
        this.dgOrders.Columns[2].FooterText = string.Format("{0:c}", Totals);

        this.dgOrders.DataBind();

        if (Anthem.Manager.IsCallBack)
            this.dgOrders.UpdateAfterCallBack = true;
    }

    protected void btnRefresh_Click(object sender, EventArgs e)
    {
        this.BindCustomerList();
        this.lstCustomers.SelectedValue = Request.Form[this.lstCustomers.UniqueID];

        if (Anthem.Manager.IsCallBack)
            this.lstCustomers.UpdateAfterCallBack = true;
    }
    protected void lstCustomers_SelectedIndexChanged(object sender, EventArgs e)
    {
        this.BindCustomerInfo(this.lstCustomers.SelectedValue);

        this.BindInvoiceList(this.lstCustomers.SelectedValue);
    }
    protected void btnSave_Click(object sender, EventArgs e)
    {
        string CustId = this.lstCustomers.SelectedValue;

        if (!Customer.Load(CustId))
        {
            this.ShowMessage("Unable load customer:\r\n" + Customer.ErrorMessage);
            this.CustomerInfoUpdatePanel.UpdateAfterCallBack = true;
            return;
        }
       
        // *** Unbind Data explicitly
        Customer.Entity.Contactname = this.txtContactName.Text;
        Customer.Entity.Companyname = this.txtCompany.Text;
        Customer.Entity.Address = this.txtAddress.Text;
        Customer.Entity.City = this.txtCity.Text;
        Customer.Entity.Region = this.txtRegion.Text;
        Customer.Entity.Postalcode = this.txtPostalCode.Text;
        Customer.Entity.Country = this.txtCountry.Text;

        if (!this.Customer.Validate())
        {
            this.ShowMessage(this.Customer.ValidationErrors.ToHtml());
            this.CustomerInfoUpdatePanel.UpdateAfterCallBack = true;
            return;
        }

        if (!this.Customer.Save())
            new ApplicationException("Unable to save customer:\r\n" +
                                     this.Customer.ErrorMessage);

        this.ShowMessage("Customer Saved.");

        // *** Rebind the Customer List
        this.BindCustomerList();

        if (Anthem.Manager.IsCallBack)
        {
            this.CustomerInfoUpdatePanel.UpdateAfterCallBack = true;
            this.lstCustomers.UpdateAfterCallBack = true;
        }
    }

    protected void ShowMessage(string Message)
    {
        this.ErrorMessage.Visible = true;
        this.ErrorMessage.Text = Message;
    }

}
