using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Westwind.BusinessObjects;
using ScriptCallbacks.BusinessObjects;
using System.Collections.Generic;

/// <summary>
/// Summary description for busStockData
/// </summary>
public class busStockUser : wwBusiness<StockusersRow>
{
	public busStockUser()
	{
        this.ConnectionString = App.Configuration.ApplicationConnectionString;
        this.ConnectType = ServerTypes.SqlServer;
        this.DataRowUpdateMode = DataRowUpdateModes.OptimizedByPk;
        this.NewRowBlankValues = true;

        this.Tablename = "StockUsers";
        this.PkField = "pk";
        this.PkType = PkFieldTypes.intType;

        // *** Don't update PK field - AutoInc value
        this.DataRowUpdateFieldFilter = "Pk";
	}

    public override object CreateNewId()
    {
        // *** Auto Inc    
        return 0;
    }

    /// <summary>
    /// Loads a customer object by Email Address
    /// </summary>
    /// <param name="Email"></param>
    /// <returns></returns>
    public bool LoadByEmail(string Email)
    {
        IDbCommand Command = this.CreateCommand("select * from " + this.Tablename + " where email=@Email");
        Command.Parameters.Add(this.CreateParameter("@Email",Email) );
        return this.LoadBase(Command);
    }
}

public class busStockData : wwBusiness<StockdataRow>
{
    public busStockData()
    {
        this.ConnectionString = App.Configuration.ApplicationConnectionString;
        this.ConnectType = ServerTypes.SqlServer;
        this.DataRowUpdateMode = DataRowUpdateModes.OptimizedByPk;
        this.NewRowBlankValues = true;
        this.Tablename = "StockData";
        this.PkField = "pk";
        this.PkType = PkFieldTypes.intType;

        // *** Don't update PK field - AutoInc value
        this.DataRowUpdateFieldFilter = "Pk";

    }

    public override object CreateNewId()
    {
        // *** Auto Inc    
        return 0;
    }

    /// <summary>
    /// Loads by UserPk and Symbol
    /// </summary>
    /// <param name="UserPk"></param>
    /// <param name="Symbol"></param>
    /// <returns></returns>
    public bool LoadByItemPkAndSymbol(int UserPk, string Symbol)
    {
        IDbCommand Command = this.CreateCommand("select * from " + this.Tablename + " where UserPk=@UserPk and Symbol=@Symbol");
        Command.Parameters.Add( this.CreateParameter("@UserPk",UserPk) );
        Command.Parameters.Add( this.CreateParameter("@Symbol",Symbol) );
        return this.LoadBase(Command);
    }

    /// <summary>
    /// Loads a data record by email address and symbol
    /// </summary>
    /// <param name="Email"></param>
    /// <param name="Symbol"></param>
    /// <returns></returns>
    public bool LoadByEmailAndSymbol(string Email, string Symbol)
    {
        return this.LoadBase("select Stockdata.* from StockData,StockUsers where StockUsers.Email=@Email and Symbol=@Symbol and UserPk = StockUsers.Pk",
                             this.CreateParameter("@Email", Email), this.CreateParameter("@Symbol", Symbol));
    }


    public int GetStockDataForUser(string Email, string Tablename)
    {
        return this.Execute(
@"select StockData.* 
FROM StockData INNER JOIN
     StockUsers ON StockData.UserPk = StockUsers.pk
WHERE StockUsers.Email = @Email", Tablename, this.CreateParameter("@Email", Email));
    }


    public string[] GetSymbols(string Email)
    {
        if (this.GetStockDataForUser(Email, "TStockData") < 0)
            return null;

        List<string> List = new List<string>();
        foreach (DataRow row in this.DataSet.Tables["TStockData"].Rows)
        {
            List.Add(row["Symbol"] as string);
        }
        return List.ToArray();
    }

    public bool ClearStockData(int Days)
    {
       int Affected = this.ExecuteNonQuery("delete " + this.Tablename + " where LastQuoteTime < @Time ",
                             this.CreateParameter("@Time",DateTime.Now.AddDays(Days * -1) ) );
        return this.Error;
    }

}