using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

/// <summary>
/// Control that implements the ICallbackEventHandler interface
/// You can use this class to hookup multiple callbacks per page,
/// one control for each callback.
/// 
/// Use the ScriptCallback event to route the event firing on
/// the control back to your page.
/// </summary>
public class EventControl : Control, ICallbackEventHandler
{
   /// <summary>
   /// Allow subscribing to the ScriptCallback event
   /// so that you can 'route' processing to another
   /// control or most commonly your Page class
   /// </summary>
   public event delScriptCallBack ScriptCallback;

   string eventArgument = null;

   void ICallbackEventHandler.RaiseCallbackEvent(string eventArgument)
   {
      this.eventArgument = eventArgument;
   }

   string ICallbackEventHandler.GetCallbackResult()
   {
      if (this.ScriptCallback != null)
         return this.ScriptCallback(this.eventArgument);

      return "Not Implemented";
   }

}

/// <summary>
/// Delegate that mirrors RaiseCallbackEvent signature
/// </summary>
public delegate string delScriptCallBack(string eventArgument);

