using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using System.Net;
using System.Text;
using System.Globalization;

using System.IO;
using System.Collections.Generic;
using Microsoft.Office.Interop.Owc11;

public class StockServer
{
    
    public StockQuote GetStockQuote(string Symbol)
    {
        WebClient Http = new WebClient();
        string QuoteString = Http.DownloadString("http://finance.yahoo.com/d/quotes.csv?s=" + Symbol + "&f=sl1d1t1c1ohgv1n");
        StockQuote Quote = this.ParseStockQuote(QuoteString);
        return Quote;
    }

    /// <summary>
    /// 
    /// </summary>
    /// <param name="Symbols"></param>
    /// <returns></returns>
    public StockQuote[] GetStockQuotes(string[] Symbols)
    {
        string Url = "http://finance.yahoo.com/d/quotes.csv?f=sl1d1t1c1ohgv1n";

        WebClient Http = new WebClient();
        foreach (string Symbol in Symbols)
        {
            Url += "&s=" + Symbol;
        }

        string RawQuoteString = Http.DownloadString(Url);

        // *** Break up into each individual quote
        string[] QuoteStrings = RawQuoteString.Split(new string[1] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);

        // *** Break up each quote CSV component and parse it into object
        StockQuote[] Quotes = new StockQuote[QuoteStrings.Length];

        for (int x = 0; x < QuoteStrings.Length; x++)
        {
            Quotes[x] = this.ParseStockQuote(QuoteStrings[x]);
        }

        return Quotes;
    }

    /// <summary>
    /// Returns the stock history for the last year
    /// http://itable.finance.yahoo.com/table.csv?s=MSFT&g=m&q=q&a=0&b=1&c=2005&d=3&e=10&&f=2006&
    /// </summary>
    /// <param name="Symbol"></param>
    /// <returns></returns>
    public StockHistory[] GetStockHistory(string Symbol, int Years)
    {
        string Url = "http://itable.finance.yahoo.com/table.csv?s=" + Symbol + "&g=m&q=q";

        DateTime Now = DateTime.Now;
        DateTime Start = Now.AddMonths((Years * 12 + 1) * -1);

        Url = Url + "&a=" + Start.Month.ToString() + "&b=" + Start.Day.ToString() + "&c=" + Start.Year.ToString();
        Url = Url + "&d=" + Now.Month.ToString() + "&e=" + Now.Day.ToString() + "&f=" + Now.Year.ToString();

        WebClient Http = new WebClient();
        // CSV String: 
        string RawQuoteString = Http.DownloadString(Url);

        string[] QuoteStrings = RawQuoteString.Split(new char[1] { '\n' }, StringSplitOptions.RemoveEmptyEntries);

        // *** We'll skip over the first line returned (CVS Header)
        StockHistory[] History = new StockHistory[QuoteStrings.Length - 1];

        // *** Note starting at 1 to skip over header 
        // *** Reading backwards to return in chronological order
        int Counter = 0;
        for (int x = QuoteStrings.Length - 1; x > 0; x--)
        {
            string[] Detail = QuoteStrings[x].Split(',');
            History[Counter] = new StockHistory();
            History[Counter].Symbol = Symbol;
            History[Counter].DateString = Detail[0].Replace("\"", "");
            History[Counter].DateString = History[Counter].DateString.Substring(History[Counter].DateString.IndexOf('-') + 1);

            decimal WorkNumber = 0.0M;
            decimal.TryParse(Detail[6], out WorkNumber);
            History[Counter].LastPrice = WorkNumber;
            Counter++;
        }

        return History;
    }

    /// <summary>
    /// This method retrieves Stock History for several symbols
    /// and then generates a chart into a specified directory
    /// The method returns a link to the image generated
    /// </summary>
    /// <param name="Symbols"></param>
    /// <returns></returns>
    public string CreateStockHistoryGraph(string[] Symbols, string Title, int Width, int Height, int Years, string ImagePath)
    {
        List<StockHistory> Histories = new List<StockHistory>();

        string FileName = "sh_" + Guid.NewGuid().GetHashCode().ToString("x") + ".png";
        string PhysicalPath = HttpContext.Current.Server.MapPath(ImagePath);
        if (!PhysicalPath.EndsWith("\\"))
            PhysicalPath += "\\";

        WebGraphing Graph = new WebGraphing();
        Graph.OutputFile = PhysicalPath + FileName;
        Graph.BackgroundColor = "LightYellow";
        Graph.Title = Title;

        Graph.Width = Width;
        Graph.Height = Height;

        // *** Start by collecting all Histories for symbols specified
        bool FirstPass = true;
        string[] Labels = null;
        foreach (string Symbol in Symbols)
        {
            StockHistory[] History = this.GetStockHistory(Symbol, Years);

            // *** Have to grab labels
            if (FirstPass)
            {
                FirstPass = false;
                Labels = new string[History.Length];
                for (int x = 0; x < History.Length; x++)
                {
                    Labels[x] = History[x].DateString;
                }
            }

            object[] Values = new object[History.Length];
            for (int x = 0; x < History.Length; x++)
            {
                Values[x] = History[x].LastPrice;
            }

            Graph.AddSeries(Symbol, Values);
        }

        Graph.ChartType = ChartChartTypeEnum.chChartTypeLineMarkers;
        Graph.CreateChart(Labels);

        // *** Do directory cleanup - cleanout files older than 10 minutes
        string[] Files = Directory.GetFiles(PhysicalPath, "sh_*.*", SearchOption.TopDirectoryOnly);
        foreach (string File in Files)
        {
            FileInfo FInfo = new FileInfo(File);
            if (FInfo.LastWriteTime < DateTime.Now.AddMinutes(-10.00))
            {
                try
                {
                    FInfo.Delete();
                }
                catch { ;}
            }
        }

        return ImagePath + "/" + FileName;
    }

    public StockQuote[] TestGetQuotes()
    {
        //return this.GetStockQuotes(new string[3] { "MSFT", "HPQ", "INTC" });
        this.CreateStockHistoryGraph(new string[3] { "MSFT", "HPQ", "INTC" }, "Portfolio History", 400, 300, 2, "temp/");
        return null;
    }

    public string TimeUntilClose()
    {
        DateTime Now = DateTime.Now;
        if (Now.Hour < 9 || Now.Hour > 17)
            return "Market is closed";

        DateTime Close = new DateTime(Now.Year, Now.Month, Now.Day, 17, 0, 0);
        TimeSpan ts = Close.Subtract(Now);

        return ts.Hours.ToString() + " " + ts.Minutes.ToString() + " until close.";
    }

    /// <summary>
    /// Parses an individual QUote String
    /// </summary>
    /// <param name="QuoteString"></param>
    /// <returns></returns>
    private StockQuote ParseStockQuote(string QuoteString)
    {
        // "MSFT",27.17,"3/10/2006","4:00pm",+0.17,27.04,27.22,26.88,-,"MICROSOFT CP"
        StockQuote Quote = new StockQuote();

        string[] Details = QuoteString.Split(',');

        Quote.Symbol = Details[0].Replace("\"", "");
        Quote.Company = Details[9].Replace("\"", "");

        string Work = Details[1];
        decimal WorkNumber = 0M;
        decimal.TryParse(Work, out WorkNumber);
        Quote.LastPrice = WorkNumber;

        Work = Details[5];
        WorkNumber = 0M;
        decimal.TryParse(Work, out WorkNumber);
        Quote.OpenPrice = WorkNumber;

        Work = Details[4];
        WorkNumber = 0.00M;
        decimal.TryParse(Work, out WorkNumber);
        Quote.NetChange = WorkNumber;

        Work = Details[2] + " " + Details[3];
        Work = Work.Replace("\"", "");
        DateTime WorkDate = DateTime.UtcNow;
        DateTime.TryParse(Work, out WorkDate); // CultureInfo.GetCultureInfo("en-us"), DateTimeStyles.AssumeLocal, out WorkDate);
        Quote.LastQuoteTime = WorkDate;

        return Quote;
    }
}

public class StockHistory
{
    public string Symbol
    {
        get { return _Symbol; }
        set { _Symbol = value; }
    }
    private string _Symbol = "";

    public decimal LastPrice
    {
        get { return _LastPrice; }
        set { _LastPrice = value; }
    }
    private decimal _LastPrice = 0.00M;

    public string DateString
    {
        get { return _DateString; }
        set { _DateString = value; }
    }
    private string _DateString = "";
}

public class StockQuote
{
    public string Symbol
    {
        get { return _Symbol; }
        set { _Symbol = value; }
    }
    private string _Symbol = "";

    public string Company
    {
        get { return _Company; }
        set { _Company = value; }
    }
    private string _Company = "";

    public decimal OpenPrice
    {
        get { return _OpenPrice; }
        set { _OpenPrice = value; }
    }
    private decimal _OpenPrice = 0.00M;


    public decimal LastPrice
    {
        get { return _LastPrice; }
        set { _LastPrice = value; }
    }
    private decimal _LastPrice = 0.00M;


    public decimal NetChange
    {
        get { return _NetChange; }
        set { _NetChange = value; }
    }
    private decimal _NetChange = 0.00M;

    public DateTime LastQuoteTime
    {
        get { return _LastQuoteTime; }
        set { _LastQuoteTime = value; }
    }
    private DateTime _LastQuoteTime = DateTime.MinValue;

    public string LastQuoteTimeString
    {
        get { return LastQuoteTime.ToString("MMM d, h:mmtt"); }
        set { _LastQuoteTimeString = value; }
    }
    private string _LastQuoteTimeString = "";
}


