using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Microsoft.Office.Interop.Owc11;
using System.IO;
using System.Text;
using System.Collections.Generic;

/// <summary>
/// This sample requires the Office Web Components 11
/// 
/// You can download them from:
/// http://www.microsoft.com/downloads/details.aspx?FamilyID=7287252c-402e-4f72-97a5-e0fd290d4b76&DisplayLang=en
/// </summary>


/// <summary>
/// Simple wrapper class around Office Web Components for simple charting needs
/// 
/// Note:
/// This is not a complete wrapper but merely a simplified access mechanism so a graph
/// can be created with a few simple property settings.
/// </summary>
public class WebGraphing
{
    
    public bool HasLegend
    {
        get { return _HasLegend; }
        set { _HasLegend = value; }
    }
    private bool _HasLegend = true;


    public string OutputFile
    {
        get { return _OutputFile; }
        set { _OutputFile = value; }
    }
    private string _OutputFile = "temp/graph.png";
    

    public int Width
    {
        get { return _Width; }
        set { _Width = value; }
    }
    private int _Width = 600;

    
    public int Height
    {
        get { return _Height; }
        set { _Height = value; }
    }
    private int _Height = 400;


    public string Title
    {
        get { return _Title; }
        set { _Title = value; }
    }
    private string _Title = "";

    
    public string BackgroundColor
    {
        get { return _BackgroundColor; }
        set { _BackgroundColor = value; }
    }
    private string _BackgroundColor = "LightYellow";
            
    public string ForeColor
    {
        get { return _ForeColor; }
        set { _ForeColor = value; }
    }
    private string _ForeColor = "Black";

    
    public string XAxisCaption
    {
        get { return _XAxisCaption; }
        set { _XAxisCaption = value; }
    }
    private string _XAxisCaption = "";

    
    public string YAxisCaption
    {
        get { return _YAxisCaption; }
        set { _YAxisCaption = value; }
    }
    private string _YAxisCaption = "";

    
    public ChartChartTypeEnum ChartType
    {
      get { return _ChartType; }
      set { _ChartType = value; }
    }
    private ChartChartTypeEnum _ChartType = ChartChartTypeEnum.chChartTypeLineMarkers;


    List<ChartSeries> Series = new List<ChartSeries>();

    public WebGraphing()
	{
    }

    /// <summary>
    /// Pass in an array of DataPoint Captions and Arrays of DataPoints
    /// </summary>
    /// <param name="Caption"></param>
    /// <param name="DataArgs"></param>
    /// <returns></returns>
    public bool CreateChart(params string[] Captions)
    {
        ChartSpace ChartSpace = new ChartSpaceClass();
        ChChart Chart = ChartSpace.Charts.Add(0);

        Chart.Type = this.ChartType;

        Chart.HasTitle = true;
        Chart.Title.Font.Bold = true;
        Chart.Title.Font.Size = 14;
        Chart.Title.Caption = this.Title;

        Chart.Axes[0].HasTitle = true;
        Chart.Axes[0].Title.Caption = this.XAxisCaption;

        Chart.Axes[1].HasTitle = true;
        Chart.Axes[1].Title.Caption = this.YAxisCaption;

        // *** Add the Captions
        StringBuilder sb = new StringBuilder();
        foreach (string Caption in Captions)
        {
            sb.Append(Caption + "\t");
        }
        string Labels = sb.ToString();

        for (int x = 0; x < this.Series.Count; x++)
        {
            ChSeries TSeries = Chart.SeriesCollection.Add(x);
            if (x == 0)
                TSeries.SetData(ChartDimensionsEnum.chDimCategories, (int)ChartSpecialDataSourcesEnum.chDataLiteral, Labels);

            ChartSeries TChartSeries = this.Series[x];
            TSeries.Interior.Color = TChartSeries.Color;
            TSeries.Caption = TChartSeries.Caption;

            sb = new StringBuilder();
            foreach (object DataPoint in TChartSeries.DataPoints)
            {
                sb.Append(DataPoint.ToString() + "\t");
            }

            TSeries.SetData(ChartDimensionsEnum.chDimValues, (int)ChartSpecialDataSourcesEnum.chDataLiteral, sb.ToString());
        }
        Chart.PlotArea.Interior.Color = this.BackgroundColor;

        if (this.Series.Count < 2)
            Chart.HasLegend = false;
        else
        {
            Chart.HasLegend = true;
        }

        string Extension = new FileInfo(OutputFile).Extension.Substring(1);

        ChartSpace.ExportPicture(this.OutputFile, Extension, this.Width, this.Height);

        return true;
    }

    public void AddSeries(string Caption, object[] DataPoints)
    {
        this.AddSeries(Caption, "", DataPoints);
    }

    public void AddSeries(string Caption, string Color, object[] DataPoints)
    {
        ChartSeries SingleSeries = new ChartSeries();
        SingleSeries.Caption = Caption;
        SingleSeries.Color =Color;
        SingleSeries.DataPoints = DataPoints;

        this.Series.Add(SingleSeries);
    }
}

public class ChartSeries
{
    public string Caption
    {
        get { return _Caption; }
        set { _Caption = value; }
    }
    private string _Caption = "";

    
    public string Color
    {
        get { return _Color; }
        set { _Color = value; }
    }
    private string _Color = "";

    
    public object[] DataPoints
    {
        get { return _DataPoints; }
        set { _DataPoints = value; }
    }
    private object[] _DataPoints = null;


}


