using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using ScriptCallbacks.BusinessObjects;

public partial class CustomerListUpdatePanel : System.Web.UI.Page
{
    protected busCustomer Customer = null;

    protected override void OnInit(EventArgs e)
    {
        base.OnInit(e);

        this.Customer = new busCustomer();
        this.BindCustomerList();
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!this.IsPostBack && this.lstCustomers.Items.Count > 0)
        {
            this.lstCustomers.SelectedValue = this.lstCustomers.Items[0].Value;
            this.BindCustomerInfo(this.lstCustomers.Items[0].Value);
        }
    }


    #region Handle SelectedValue with OnPreRender and LoadViewState method overloads
    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);
        this.ViewState["lstCustomers_SelectedValue"] = this.lstCustomers.SelectedValue;
    }

    protected override void LoadViewState(object savedState)
    {
        base.LoadViewState(savedState);

        string Val = this.ViewState["lstCustomers_SelectedValue"] as string;
        if (Val != null)
            this.lstCustomers.SelectedValue = Val;
    }
    #endregion

    protected void lstCustomers_SelectedIndexChanged(object sender, EventArgs e)
    {
        string CustomerId = this.lstCustomers.SelectedValue;
        this.BindCustomerInfo(this.lstCustomers.SelectedValue);

        // *** Have to rebind the list value
//        this.lstCustomers.SelectedValue = CustomerId;
    }

    protected void btnSave_Click(object sender, EventArgs e)
    {
        this.SaveCustomer();    

        // *** Reselect item
        int Selected = this.lstCustomers.SelectedIndex;

        // *** Reload and bind Customer List
        // *** if the Company has changed
        this.BindCustomerList();

        this.lstCustomers.SelectedIndex = Selected;

        this.CustListUpdatePanel.Update();
    }

    public void BindCustomerInfo(string CustomerId)
    {
        // Load the customer object and Entity member
        this.Customer.Load(CustomerId);

        // *** Rebind only the Panel! Don't bind whole form
        // *** or there will be problems with the List selectedvalue
        this.CustomerUpdatePanel.DataBind();
    }


    public void BindCustomerList()
    {
        if (this.Customer.GetCustomerList() < 0)
        {
            this.txtCompany.Text = this.Customer.ErrorMessage;
            this.txtAddress.Text = Environment.UserName;
            return;
        }

        this.lstCustomers.DataSource = Customer.DataSet.Tables["TCustomerList"];
        this.lstCustomers.DataValueField = "CustomerId";
        this.lstCustomers.DataTextField = "CompanyName";
        this.lstCustomers.DataBind();
    }


    /// <summary>
    /// Demonstrates using POSTback form vars
    /// </summary>
    /// <param name="CustomerId"></param>
    /// <returns></returns>
    public bool SaveCustomer()
    {
        string CustomerId =  this.lstCustomers.SelectedValue;

        this.Customer = new busCustomer();

        if (CustomerId == null)
            throw new ApplicationException("Invalid CustomerId");
        
        if (CustomerId == "")
        {
            if (!Customer.New())
                throw new ApplicationException("Couldn't create new customer.");
        }
        else
        {
            if (!Customer.Load(CustomerId))
                throw new ApplicationException("Couldn't load customer");
        }

        // *** Pick up values that were posted back from the
        // *** the client callback
        Customer.Entity.Contactname = this.txtContactName.Text;
        Customer.Entity.Companyname = this.txtCompany.Text;
        Customer.Entity.Address = this.txtAddress.Text;
        Customer.Entity.City = this.txtCity.Text;

        if (!Customer.Validate())
            throw new ApplicationException(Customer.ValidationErrors.ToString());
  
        if (!Customer.Save())
            throw new Exception(Customer.ErrorMessage);

        return true;
    }


}
