using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Threading;

using ScriptCallbacks.BusinessObjects;
using Westwind.Tools;
using System.Web.Services;


public partial class Atlas_LongProgress : System.Web.UI.Page
{

    protected string RequestKey = "";

    protected void Page_Load(object sender, EventArgs e)
    {
    }

    /// <summary>
    /// Client called WebMethod that starts the request processing
    /// sequence. Returns a RequestKey to the client that the
    /// client must pass to all callbacks to identify which messages
    /// to look at.
    /// 
    /// This routine generates a new RequestKey and spins up a new thread
    /// for processing the task. 
    /// </summary>
    /// <returns></returns>
    [WebMethod]
    public string StartRequestProcessing()
    {
        this.RequestKey = Guid.NewGuid().GetHashCode().ToString("x");

        busQueueMessage Message = new busQueueMessage();
        Message.New();

        Message.Entity.Reqid = this.RequestKey;
        Message.Entity.Message = "Data Retrieval started...";
        Message.Entity.Percentcomplete = 1;
        Message.Save();

        Thread Thd = new Thread( new  ParameterizedThreadStart(this.Process));
        Thd.Start(this.RequestKey);

        return RequestKey;
    }

    /// <summary>
    /// Used to cancel a processing request. This basically writes CANCEL
    /// message to the message table - it's up to the processing engine
    /// to actually check for this Cancel message.
    /// </summary>
    /// <param name="RequestKey"></param>
    /// <returns></returns>
    [WebMethod]
    public bool CancelRequestProcessing(string RequestKey)
    {
        busQueueMessage Message = new busQueueMessage();

        if (!Message.Load(RequestKey))
            return false;

        Message.CancelRequest();
        Message.Entity.Message = "Request cancelled by user...";
        Message.Save();

        return true;
    }

    /// <summary>
    /// This method is called to get status updates. It returns a custom
    /// object that is a subset of the full Message.Entity in order to 
    /// minimize the amount of data sent back. Message contains Message,
    /// Status and Percent value - just what the client needs.
    /// </summary>
    /// <param name="RequestKey"></param>
    /// <returns></returns>
    [WebMethod]
    public UpdateStatus GetRequestStatus(string RequestKey)
    {
        busQueueMessage Message = new busQueueMessage();

        if (!Message.Load(RequestKey))
            return null;

        UpdateStatus Status = new UpdateStatus();
        Status.Message = Message.Entity.Message;
        Status.Status = Message.Entity.Status;
        Status.Percent = Message.Entity.Percentcomplete;

        return Status;
    }


    /// <summary>
    /// This button click is fired as a Callback from the client within
    /// an update panel. It's simply used to bind and display the datagrid
    /// and hide the ProgressContainer.
    /// 
    /// NOTE: You shouldn't set the container to visible=false - if you do
    /// the control gets wiped off the page and you can't re-run the query
    /// and display the progress bar.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnComplete_Click(object sender, EventArgs e)
    {
        string RequestKey = this.txtRequestKey.Value;

        busQueueMessage Message = new busQueueMessage();
        if (!Message.Load(RequestKey))
            return;

        if (Message.Entity.Status != "Completed")
            return;

        // *** Important: Don't set visible to false or else
        //                you can't restart the progress display
        this.ProgressContainer.Style.Add("display", "none");

        DataTable Data = wwUtils.DeSerializeObject(Message.Entity.Bincontent, 
                                                   typeof(DataTable)) as DataTable;

        this.gdCustomers.DataSource = Data;
        this.gdCustomers.DataBind();
        

        // *** Message cleanup for 'old' messages
        Message.ClearMessages();

    }


    /// <summary>
    /// This is the long worker method. In this case we're simply looping 100 times
    /// and then in the end running a query to retrieve some data and store that in 
    /// the ChatMessages table. The result is then displayed on the Web page via
    /// ATLAS server control updates.
    /// 
    /// While spinning the Message object is updated with status information. This
    /// information is picked up by the clients who are calling the GetRequestStatus()
    /// [WebMethod] function. This function updates the client which in turn sets the 
    /// progress bar percentage and message.
    /// 
    /// When done this method runs a real query and stores the result into a DataTable.
    /// The table is serialized and stored in the Message table's BinContent field.
    /// 
    /// When the btnComplete event is fired that request picks up this serialized
    /// DataTable and binds it to the datagrid.
    /// </summary>
    protected void Process(object oRequestKey)
    {
        string RequestKey = (string)oRequestKey;

        busQueueMessage Message = new busQueueMessage();

        for (int x = 1; x < 100; x++)
        {
            Message.Load(RequestKey);

            if (Message.Entity.Status == "Cancelled")
            {
                Message.CancelRequest();
                Message.Entity.Message = "Processing has been cancelled on the server...";
                Message.Save();
                return;
            }

            Thread.Sleep(200);
            
            Message.Entity.Percentcomplete = x;
            Message.Entity.Message = "Task #" + x.ToString() + " is now processing...";
            Message.Save();
        }

        // *** Simulate a real result
        busCustomer Customer = new busCustomer();
        Customer.GetCustomerList();

        byte[] Buffer = null;
        wwUtils.SerializeObject(Customer.DataSet.Tables["TCustomerList"], out Buffer);
        Message.Entity.Bincontent = Buffer;

        Message.CompleteRequest();
        Message.Save();
    }


    public class UpdateStatus
    {
        
        public string Status
        {
          get { return _Status; }
          set { _Status = value; }
        }
        private string _Status = "";
        
        
        public int Percent
        {
          get { return _Percent; }
          set { _Percent = value; }
        }
        private int _Percent = 0;

        public string Message
        {
          get { return _Message; }
          set { _Message = value; }
        }
        private string _Message = "";

    }

}
