using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Microsoft.Office.Interop.Owc11;
using System.Drawing;
using Westwind.Tools;


/// <summary>
/// This sample requires the Office Web Components 11
/// 
/// You can downlaod them from:
/// http://www.microsoft.com/downloads/details.aspx?FamilyID=7287252c-402e-4f72-97a5-e0fd290d4b76&DisplayLang=en
/// 
/// </summary>

public partial class StockDisplay : System.Web.UI.Page
{
    /// <summary>
    /// The Email/ID for the user we use to attach to the user
    /// </summary>
    protected string UserEmail = "";

    protected void Page_Load(object sender, EventArgs e)
    {
        // *** Try to load the users email address
        this.GetUserEmail();

        if (!this.IsPostBack)
            this.RefreshPortfolio();

        if (!this.ScriptManager.IsInPartialRenderingMode)
            this.CloseTimeTimer_Tick(this, EventArgs.Empty);

    }
   
    /// <summary>
    /// Update the 
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void CloseTimeTimer_Tick(object sender, EventArgs e)
    {
        DateTime Now = DateTime.Now;
        if (Now.Hour < 9 || Now.Hour > 17)
        {
            this.lblCloseTime.Text = "The market is closed.";
            return;
        }

        DateTime Close = new DateTime(Now.Year, Now.Month, Now.Day, 17, 0, 0);
        TimeSpan ts = Close.Subtract(Now);

        this.lblCloseTime.Text = ts.Hours.ToString() + " hrs " + ts.Minutes.ToString() + " mins until close.";
    }

    protected void chkGraph_CheckedChanged(object sender, EventArgs e)
    {
        StockServer Stock = new StockServer();
        busStockData StockData = new busStockData();
        
        string[] SymbolList = StockData.GetSymbols(this.UserEmail);
     
        if (this.chkGraph.Checked)
        {
            string HistoryImage = Stock.CreateStockHistoryGraph(SymbolList, "2 year history", 680, 400, 2, "temp");
            this.imgHistory.ImageUrl = HistoryImage;
            this.imgHistory.Visible = true;
        }
        else
            this.imgHistory.Visible = false;

    }

    protected void btnRefresh_Click(object sender, EventArgs e)
    {
        this.RefreshPortfolio();       
    }

    protected void RefreshPortfolio()
    {
        StockServer Stock = new StockServer();

        if (this.UserEmail == "")
            return;

        busStockData StockData = new busStockData();
        
        int Result = StockData.GetStockDataForUser(this.UserEmail, "TStockData");

        if (Result < 1)
        {
            this.ErrorDisplay.ShowError("Invalid email or portfolio is empty.");
            this.grdStocks.Visible = false;
            return;
        }
        this.grdStocks.Visible = true;
            
        string[] SymbolList = StockData.GetSymbols(this.UserEmail);
        StockQuote[] Quotes = Stock.GetStockQuotes(SymbolList);

        decimal TotalPosition = 0.00M;

        DataTable StockTable = StockData.DataSet.Tables["TStockData"];
        for (int x = 0; x < StockTable.Rows.Count; x++ )
        {
            DataRow row = StockTable.Rows[x];
            row["LastPrice"] = Quotes[x].LastPrice;

            row["Name"] = Quotes[x].Company;
            row["LastQuoteTime"] = Quotes[x].LastQuoteTime;

            decimal Position = Quotes[x].LastPrice * (int)row["Qty"]; 
            TotalPosition += Position;
            row["Total"] = Position;
        }

        //StockData.SaveTable("TStockData");
        
        this.grdStocks.DataSource = StockTable;
        this.grdStocks.DataBind();

        TableCell FooterSummaryCell = this.grdStocks.FooterRow.Cells[this.grdStocks.FooterRow.Cells.Count - 1];
        FooterSummaryCell.HorizontalAlign = HorizontalAlign.Right;
        FooterSummaryCell.BackColor = Color.Cornsilk;
        FooterSummaryCell.ForeColor = Color.DarkBlue;
        FooterSummaryCell.Font.Bold = true;
        FooterSummaryCell.Text = TotalPosition.ToString("c");

        if (this.chkGraph.Checked)
        {
            string HistoryImage = Stock.CreateStockHistoryGraph(SymbolList, "2 year history", 680, 400, 2, "temp");
            this.imgHistory.ImageUrl = HistoryImage;
            this.imgHistory.Visible = true;
        }
        else
            this.imgHistory.Visible = false;
    }

    protected void GetUserEmail()
    {
        HttpCookie Cookie = null;
        string Email = this.txtEmail.Text;
        
        if (Email == "")
        {
             Cookie = Request.Cookies["Stock_Email"];
             if (Cookie != null)
                 Email = Cookie.Value;
         }
        else
        {
            Cookie = new HttpCookie("Stock_Email", Email);
            Cookie.Expires = DateTime.Now.AddMonths(2);
            Response.Cookies.Add(Cookie);
        }

        this.txtEmail.Text = Email;
        this.UserEmail = Email;
    }
    protected void grdStocks_RowEditing(object sender, GridViewEditEventArgs e)
    {
        grdStocks.EditIndex = e.NewEditIndex;
        this.RefreshPortfolio();
    }
    protected void grdStocks_RowUpdating(object sender, GridViewUpdateEventArgs e)
    {
        string Symbol = ((TextBox)grdStocks.Rows[e.RowIndex].Cells[1].Controls[0]).Text;
        string Qty = ((TextBox)grdStocks.Rows[e.RowIndex].Cells[4].Controls[0]).Text;

        busStockData StockData = new busStockData();
        StockData.GetStockDataForUser(this.txtEmail.Text, "TStockData");

        DataTable StockDataTable = StockData.DataSet.Tables["TStockData"];
        string OldSymbol = StockDataTable.Rows[e.RowIndex]["Symbol"] as string;
        int UserPk = (int) StockDataTable.Rows[e.RowIndex]["UserPk"];

        if (!StockData.LoadByItemPkAndSymbol(UserPk, OldSymbol))
        {
            this.ErrorDisplay.ShowError("Invalid Stock Selection");
            return;
        }

        StockData.DataRow["Qty"] = int.Parse(Qty);
        StockData.DataRow["Symbol"] = Symbol; 

        StockData.Save();
        
        grdStocks.EditIndex = -1;

        this.RefreshPortfolio();
    }
    protected void grdStocks_RowDeleting(object sender, GridViewDeleteEventArgs e)
    {
        busStockData StockData = new busStockData();
        StockData.GetStockDataForUser(this.txtEmail.Text, "TStockData");

        DataTable StockDataTable = StockData.DataSet.Tables["TStockData"];
        int ItemPk = (int)StockDataTable.Rows[e.RowIndex]["Pk"];

        if (StockData.Delete(ItemPk))
        {
            this.ErrorDisplay.ShowMessage("Stock removed.");
            this.RefreshPortfolio();
        }
        else
            this.ErrorDisplay.ShowError("Unable to remove stock.");

    }
    protected void grdStocks_RowCancelingEdit(object sender, GridViewCancelEditEventArgs e)
    {
        this.grdStocks.EditIndex = -1;
    }
    protected void btnAddStock_Click(object sender, EventArgs e)
    {
        if (this.txtEmail.Text == "")
        {
            this.ErrorDisplay.ShowMessage("Please enter an email address or handle first.");
            return;
        }
        
        busStockData StockData = new busStockData();
        StockData.GetStockDataForUser(this.txtEmail.Text, "TStockData");

        busStockUser StockUser = new busStockUser();
        if (!StockUser.LoadByEmail(this.txtEmail.Text))
        {
            if (!string.IsNullOrEmpty(this.txtEmail.Text))
            {
                StockUser.New();
                StockUser.Entity.Email = this.txtEmail.Text;
                StockUser.Save();

                // re-read so we get the PK
                StockUser.LoadByEmail(this.txtEmail.Text); 
            }
        }

        int Qty = 0;
        int.TryParse(this.txtNewStockQty.Text, out Qty);
        if (Qty == 0)
        {
            this.ErrorDisplay.ShowError("Invalid Qty provided.");
            return; 
        }
        
        StockData.New();
        StockData.Entity.Userpk = StockUser.Pk;
        StockData.Entity.Symbol = this.txtAddStockSymbol.Text;
        
        StockData.Entity.Qty = Qty; 

        StockData.Save();

        this.RefreshPortfolio();
    }
    
}
