using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Text;
using System.IO;

public partial class InvoiceList : System.Web.UI.Page, ICallbackEventHandler
{
    protected string CallBackEventReference;
    
    protected void Page_Load(object sender, EventArgs e)
    {
        // *** On an event callback you might not want to
        // *** run your page initialization code
        if (!this.IsCallback)
        {
            busInvoice Invoice = new busInvoice();
            
            int ResultCount = Invoice.GetLastInvoices(100);
            if (ResultCount < 0)
            {
                this.ShowError(Invoice.ErrorMessage);
                return;
            }

            this.gvInvoiceList.DataSource = Invoice.DataSet.Tables["TInvoiceList"].DefaultView;
            this.gvInvoiceList.DataBind();

            this.HookUpEventReference();
        }
    }

    void HookUpEventReference()
    {
       // *** Create an event reference
        this.CallBackEventReference =
            this.ClientScript.GetCallbackEventReference(
                this,
                "OrderId",
                "OnLineItemView",
                "'LineItemView'",true);
    }
    
    protected void ShowError(string Message)
    {
        this.lblErrorMessage.Text = Message;
        this.lblErrorMessage.Visible = true;

    }
    protected void gvInvoiceList_PageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        this.gvInvoiceList.PageIndex = e.NewPageIndex;
        this.gvInvoiceList.DataBind();
    }

    #region ICallbackEventHandler Members
    string eventArgument = "";

    public void RaiseCallbackEvent(string eventArgument)
    {
        this.eventArgument = eventArgument;
    }

    public string GetCallbackResult()
    {
        if (!this.chkFullInvoice.Checked)
        {
            // *** Use 'static' Html generation in business object
            int OrderId = -1;
            if (!int.TryParse(eventArgument, out OrderId))
                return "";

            // *** Create Invoice Bus Object and load Order
            busInvoice Invoice = new busInvoice();
            if (!Invoice.Load(eventArgument))
                return "";

            // *** Generate Html for lineitems
            return Invoice.HtmlLineItems();
        }
        else
        {
            // *** Call out to another ASPX page to handle 
            // *** Html generation 
            string ErrorMessage = "";
            string Output = AspTextMerge("ScriptCallbacks/InvoiceAbstract.aspx?id=" + eventArgument, out ErrorMessage);
            return Output;
        }
    }

    #endregion

    public static string AspTextMerge(string TemplatePageAndQueryString, out string ErrorMessage)
    {
        string MergedText = "";
        ErrorMessage = "";

        // *** Save the current request information
        HttpContext Context = HttpContext.Current;

        // *** Fix up the path to point at the templates directory
        TemplatePageAndQueryString = Context.Request.ApplicationPath + "/" + TemplatePageAndQueryString;

        // *** Now call the other page and load into StringWriter
        StringWriter sw = new StringWriter();
        try
        {
            // *** IMPORTANT: Child page's FilePath still points at current page
            //                QueryString provided is mapped into new page and then reset
            Context.Server.Execute(TemplatePageAndQueryString, sw, false);
            MergedText = sw.ToString();
        }
        catch (Exception ex)
        {
            MergedText = null;
            ErrorMessage = ex.Message;
        }

        return MergedText;
    }
} 
