// *** Set this flag to send the invoice list as HTML
// *** as an alternate example on how to fill the listbox of invoices
//#define HtmlInvoiceList

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web; 
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Westwind.Tools;
using System.Data.SqlClient;
using System.Text;

using ScriptCallbacks.BusinessObjects;
using Westwind.Web.Controls;

public partial class InvoiceLookup : System.Web.UI.Page
{
    public EventControl CustomerEventControl = new EventControl();
    public EventControl InvoiceEventControl = new EventControl();

    public EventControl CustomerHtmlEventControl = new EventControl();

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsCallback)
        {
            busCustomer Customer = new busCustomer();
            if (Customer.GetCustomerList() < 0)
            {
                this.ShowError("Unable to load customers: " + Customer.ErrorMessage);
                return;
            }

            this.txtCustomerPk.DataSource = Customer.DataSet.Tables["TCustomerList"].DefaultView;
            this.txtCustomerPk.DataTextField = "CompanyName";
            this.txtCustomerPk.DataValueField = "CustomerId";
            this.txtCustomerPk.DataBind();
        }
        this.SetupScriptCallbacks();
    }

    protected void SetupScriptCallbacks()
    {
    #if !HtmlInvoiceList
        // *** Add the Event controls that implement ICallbackEventHandler
        // *** and attach the ScriptCallback event to pass us notifications into this form
        this.CustomerEventControl.ScriptCallback += 
                   new delScriptCallBack(CustomerEventControl_ScriptCallback);        
        this.AddParsedSubObject(this.CustomerEventControl);

        // *** Attach the callback handler to the server event
        // *** via Client Event
        string ScriptRef = this.ClientScript.GetCallbackEventReference(
                    this.CustomerEventControl,
                    "document.forms[0].txtCustomerPk.value",
                    "OnCustomerSelectionCallback",
                    "'Customer'",
                    "OnCustomerSelectionCallback",
                    true);
        
       // *** Assign the Script Code to the onchange event
       this.txtCustomerPk.Attributes.Add("onChange",ScriptRef);
    #else
        // *** Add the Event controls that implement ICallbackEventHandler
        // *** and attach the ScriptCallback event to pass us notifications into this form
        this.CustomerHtmlEventControl.ScriptCallback += new delScriptCallBack(CustomerHtmlEventControl_ScriptCallback);
        this.AddParsedSubObject(this.CustomerHtmlEventControl);

        // *** Attach the callback handler to the server event
        // *** via Client Event
        string ScriptRef = this.ClientScript.GetCallbackEventReference(
                    this.CustomerHtmlEventControl,
                    "document.forms[0].txtCustomerPk.value",
                    "OnCustomerSelectionHtmlCallback",
                    "'Customer'",
                    "OnCustomerSelectionHtmlCallback",
                    true);
        this.txtCustomerPk.Attributes.Add("onchange", ScriptRef);
    #endif

        // *** add the Invoice Event Control so we have a separate handler
        this.InvoiceEventControl.ScriptCallback += 
                new delScriptCallBack(InvoiceEventControl_ScriptCallback);
        
        this.AddParsedSubObject(this.InvoiceEventControl);

        ScriptRef = this.ClientScript.GetCallbackEventReference(
                this.InvoiceEventControl,
                "document.forms[0].txtInvoiceList.value",
                "OnInvoiceSelectionCallback",
                "'Invoice'",
                "OnInvoiceSelectionCallback",
                true);
        this.txtInvoiceList.Attributes.Add("onchange", ScriptRef);
    }


     /// <summary>
     /// Fires in response to the customer list selection. This method
     /// retrieves a list of invoices and then builds a string list that
     /// contains the pk on one line and the display string on the second
     /// line. The client code then has to parse this string list.
     /// </summary>
     /// <param name="eventArgument"></param>
     /// <returns></returns>
    string CustomerEventControl_ScriptCallback(string eventArgument)
    {
        string CustomerId = eventArgument;

        busInvoice Invoice = new busInvoice();
        if (Invoice.GetInvoicesForCustomer(CustomerId) < 0)
            return "Error: " + Invoice.ErrorMessage;

        StringBuilder Result = new StringBuilder();

        foreach (DataRow dr in Invoice.DataSet.Tables["TCustomerInvoiceList"].Rows)
        {
            // *** Write out the PK
            Result.AppendLine( dr["Orderid"].ToString() );

            Result.AppendLine( 
                ((DateTime) dr["OrderDate"]).ToString("yyy-MM-dd") + "   #" + 
                dr["OrderId"].ToString() + "     " + 
                ( (decimal) dr["OrderTotal"] + 
                  (decimal) dr["Freight"]).ToString("c") 
                );
        }

        return Result.ToString();
    }

    /// <summary>
    /// This method is similar to the previous one, but this one returns an HTML
    /// result that contains the full <option> elements that can be embedded into
    /// the list control.
    /// 
    /// This simplifies the code on the client which can more or less replace
    /// the list content rather than having to parse the result data.
    /// </summary>
    /// <param name="eventArgument"></param>
    /// <returns></returns>
    string CustomerHtmlEventControl_ScriptCallback(string eventArgument)
    {
        string CustomerId = eventArgument;

        busInvoice Invoice = new busInvoice();
        if (Invoice.GetInvoicesForCustomer(CustomerId) < 0)
            return "Error: " + Invoice.ErrorMessage;

        StringBuilder Result = new StringBuilder();

        foreach (DataRow dr in Invoice.DataSet.Tables["TCustomerInvoiceList"].Rows)
        {
            // *** Write out the PK
            Result.Append("<option value='" + dr["Orderid"].ToString() + "'>");

            Result.Append(
                ((DateTime)dr["OrderDate"]).ToString("yyy-MM-dd") + "&nbsp;&nbsp;&nbsp;&nbsp;#" +
                dr["OrderId"].ToString() + "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
                ((decimal)dr["OrderTotal"] + (decimal)dr["Freight"]).ToString("c")
                );
            Result.Append("</option>");
        }

        return Result.ToString();
    }

    /// <summary>
    /// This method retrieves an Invoice's lineitems and returns the 
    /// result as an Html table that simply gets embedded into the 
    /// client's code by replacing the innerHTML of a placeholder <div>
    /// tag.
    /// </summary>
    /// <param name="eventArgument"></param>
    /// <returns></returns>
    string InvoiceEventControl_ScriptCallback(string eventArgument)
    {

        int InvNo = -1;
        if (!int.TryParse(eventArgument, out InvNo))
            return "Error: Invalid key passed " + this.txtInvoiceList.Text;
        
        busInvoice Invoice = new busInvoice();
        if (!Invoice.Load(InvNo))
        {
            return "Error: " + Invoice.ErrorMessage + this.txtInvoiceList.Text;
        }

        return Invoice.HtmlLineItems();
    }

    void ShowError(string Message)
    {
        this.lblErrorMessage.Text = Message  + "<p>";
        this.lblErrorMessage.Visible = true;
    }
}


